@@ -1,4 +1,4 @@
-/* $Id: info.c 3626 2008-12-16 13:05:11Z mark_ellis $ */
+/* $Id: info.c 3780 2009-06-08 16:41:06Z mark_ellis $ */
 #define _BSD_SOURCE 1
 #ifdef HAVE_CONFIG_H
 #include "synce_config.h"
@@ -10,12 +10,10 @@
 #define DBUS_API_SUBJECT_TO_CHANGE 1
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
-#include <sys/socket.h>
-#include <sys/un.h>
 #endif
 #include <sys/types.h>
-#include <unistd.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #if ENABLE_HAL_SUPPORT
@@ -23,6 +21,26 @@
 #include <libhal.h>
 #endif
 
+struct _SynceInfo
+{
+  pid_t dccm_pid;
+  char* device_ip;
+  char* local_iface_ip;
+  char* password;
+  int key;
+  int os_major;
+  int os_minor;
+  int build_number;
+  int processor_type;
+  int partner_id_1;
+  int partner_id_2;
+  char* name;
+  char* os_name;
+  char* model;
+  char* transport;
+  char* object_path;
+};
+
 #if ENABLE_ODCCM_SUPPORT
 static const char* const DBUS_SERVICE       = "org.freedesktop.DBus";
 static const char* const DBUS_IFACE         = "org.freedesktop.DBus";
@@ -77,16 +95,18 @@ static SynceInfo* synce_info_from_file(const char* device_name)
   result->dccm_pid        = getConfigInt(config, "dccm",   "pid");
 
   result->key             = getConfigInt(config, "device", "key");
-  result->os_version      = getConfigInt(config, "device", "os_version");
+  result->os_major        = getConfigInt(config, "device", "os_version");
   result->os_minor        = getConfigInt(config, "device", "os_minor");
   result->build_number    = getConfigInt(config, "device", "build_number");
   result->processor_type  = getConfigInt(config, "device", "processor_type");
   result->partner_id_1    = getConfigInt(config, "device", "partner_id_1");
   result->partner_id_2    = getConfigInt(config, "device", "partner_id_2");
 
-  result->ip        = STRDUP(getConfigString(config, "device", "ip"));
+  result->device_ip       = STRDUP(getConfigString(config, "device", "ip"));
+  result->local_iface_ip  = NULL;
   result->password  = STRDUP(getConfigString(config, "device", "password"));
   result->name      = STRDUP(getConfigString(config, "device", "name"));
+  result->object_path     = STRDUP(connection_filename);
 
   if (!(result->os_name = STRDUP(getConfigString(config, "device", "os_name"))))
           result->os_name = STRDUP(getConfigString(config, "device", "class"));
@@ -115,60 +135,6 @@ exit:
 
 #endif /* ENABLE_DCCM_FILE_SUPPORT */
 
-#if ENABLE_ODCCM_SUPPORT || ENABLE_HAL_SUPPORT
-
-gint
-get_socket_from_dccm(const gchar *unix_path)
-{
-  int fd = -1, dev_fd, ret;
-  struct sockaddr_un sa;
-  struct msghdr msg = { 0, };
-  struct cmsghdr *cmsg;
-  struct iovec iov;
-  char cmsg_buf[512];
-  char data_buf[512];
-
-  fd = socket(AF_UNIX, SOCK_STREAM, 0);
-  if (fd < 0)
-    goto ERROR;
-
-  sa.sun_family = AF_UNIX;
-  strcpy(sa.sun_path, unix_path);
-
-  if (connect(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0)
-    goto ERROR;
-
-  msg.msg_control = cmsg_buf;
-  msg.msg_controllen = sizeof(cmsg_buf);
-  msg.msg_iov = &iov;
-  msg.msg_iovlen = 1;
-  msg.msg_flags = MSG_WAITALL;
-
-  iov.iov_base = data_buf;
-  iov.iov_len = sizeof(data_buf);
-
-  ret = recvmsg(fd, &msg, 0);
-  if (ret < 0)
-    goto ERROR;
-
-  cmsg = CMSG_FIRSTHDR (&msg);
-  if (cmsg == NULL || cmsg->cmsg_type != SCM_RIGHTS)
-    goto ERROR;
-
-  dev_fd = *((int *) CMSG_DATA(cmsg));
-  goto OUT;
-
-ERROR:
-  dev_fd = -1;
-
-OUT:
-  if (fd >= 0)
-    close(fd);
-
-  return dev_fd;
-}
-#endif
-
 #if ENABLE_ODCCM_SUPPORT
 
 #define ODCCM_TYPE_OBJECT_PATH_ARRAY \
@@ -275,7 +241,6 @@ static SynceInfo *synce_info_from_odccm(const char* device_name)
       goto ERROR;
     }
 
-    gchar *unix_path;
     guint os_major;
     guint os_minor;
     gchar* ip;
@@ -284,6 +249,7 @@ static SynceInfo *synce_info_from_odccm(const char* device_name)
     gchar* model;
 
     result->name = name;
+    result->object_path = g_strdup(obj_path);
 
     if (!dbus_g_proxy_call(proxy, "GetOsVersion", &error,
                            G_TYPE_INVALID,
@@ -295,7 +261,7 @@ static SynceInfo *synce_info_from_odccm(const char* device_name)
       g_object_unref(proxy);
       goto ERROR;
     }
-    result->os_version = os_major;
+    result->os_major = os_major;
     result->os_minor = os_minor;
 
     if (!dbus_g_proxy_call(proxy, "GetIpAddress", &error,
@@ -307,7 +273,8 @@ static SynceInfo *synce_info_from_odccm(const char* device_name)
       g_object_unref(proxy);
       goto ERROR;
     }
-    result->ip = ip;
+    result->device_ip = ip;
+    result->local_iface_ip = NULL;
 
     if (!dbus_g_proxy_call(proxy, "GetCpuType", &error,
                            G_TYPE_INVALID,
@@ -342,27 +309,8 @@ static SynceInfo *synce_info_from_odccm(const char* device_name)
     }
     result->model = model;
 
-    if (!dbus_g_proxy_call(proxy, "RequestConnection", &error,
-                           G_TYPE_INVALID,
-                           G_TYPE_STRING, &unix_path,
-                           G_TYPE_INVALID))
-    {
-      g_warning("%s: Failed to get a connection for %s: %s", G_STRFUNC, result->name, error->message);
-      g_object_unref(proxy);
-      goto ERROR;
-    }
-
     g_object_unref(proxy);
 
-    result->fd = get_socket_from_dccm(unix_path);
-    g_free(unix_path);
-
-    if (result->fd < 0)
-    {
-      g_warning("%s: Failed to get file-descriptor from odccm for %s", G_STRFUNC, result->name);
-      goto ERROR;
-    }
-
     result->transport = g_strdup("odccm");
 
     break;
@@ -408,6 +356,7 @@ static SynceInfo *synce_info_from_hal(const char* device_name)
   gint i;
   gchar **device_list = NULL;
   gint num_devices;
+  gboolean disabled;
 
   g_type_init();
   dbus_error_init(&dbus_error);
@@ -448,6 +397,21 @@ static SynceInfo *synce_info_from_hal(const char* device_name)
   }
 
   for (i = 0; i < num_devices; i++) {
+
+    /* discard unused ports for 4 endpoint serial devices */
+    if (libhal_device_property_exists(hal_ctx, device_list[i], "pda.pocketpc.disabled", &dbus_error)) {
+
+            disabled = libhal_device_get_property_bool(hal_ctx, device_list[i], "pda.pocketpc.disabled", &dbus_error);
+            if (dbus_error_is_set(&dbus_error)) {
+                    g_critical("%s: Failed to obtain property pda.pocketpc.disabled for device %s: %s: %s",
+                               G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
+                    goto error_exit;
+            }
+
+            if (disabled)
+                    continue;
+    }
+
     if (!(libhal_device_property_exists(hal_ctx, device_list[i], "pda.pocketpc.name", &dbus_error))) {
             if (dbus_error_is_set(&dbus_error)) {
                     g_critical("%s: Failed to check for property pda.pocketpc.name for device %s: %s: %s", G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
@@ -480,8 +444,9 @@ static SynceInfo *synce_info_from_hal(const char* device_name)
 
     result->name = g_strdup(name);
     libhal_free_string(name);
+    result->object_path = g_strdup(device_list[i]);
 
-    result->os_version = libhal_device_get_property_uint64(hal_ctx, device_list[i], "pda.pocketpc.os_major", &dbus_error);
+    result->os_major = libhal_device_get_property_uint64(hal_ctx, device_list[i], "pda.pocketpc.os_major", &dbus_error);
     if (dbus_error_is_set(&dbus_error)) {
       g_critical("%s: Failed to obtain property pda.pocketpc.os_major for device %s: %s: %s", G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
       goto error_exit;
@@ -499,12 +464,19 @@ static SynceInfo *synce_info_from_hal(const char* device_name)
       goto error_exit;
     }
 
-    result->ip = libhal_device_get_property_string(hal_ctx, device_list[i], "pda.pocketpc.ip_address", &dbus_error);
+    result->device_ip = libhal_device_get_property_string(hal_ctx, device_list[i], "pda.pocketpc.ip_address", &dbus_error);
     if (dbus_error_is_set(&dbus_error)) {
       g_critical("%s: Failed to obtain property pda.pocketpc.ip_address for device %s: %s: %s", G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
       goto error_exit;
     }
 
+    result->local_iface_ip = libhal_device_get_property_string(hal_ctx, device_list[i], "pda.pocketpc.iface_address", &dbus_error);
+    if (dbus_error_is_set(&dbus_error)) {
+      g_warning("%s: Failed to obtain property pda.pocketpc.iface_address for device %s: %s: %s", G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
+      result->local_iface_ip = NULL;
+      dbus_error_free(&dbus_error);
+    }
+
     result->os_name = libhal_device_get_property_string(hal_ctx, device_list[i], "pda.pocketpc.platform", &dbus_error);
     if (dbus_error_is_set(&dbus_error)) {
       g_critical("%s: Failed to obtain property pda.pocketpc.platform for device %s: %s: %s", G_STRFUNC, device_list[i], dbus_error.name, dbus_error.message);
@@ -517,37 +489,6 @@ static SynceInfo *synce_info_from_hal(const char* device_name)
       goto error_exit;
     }
 
-    gchar *unix_path;
-
-    DBusGProxy *proxy = dbus_g_proxy_new_for_name(system_bus,
-						  "org.freedesktop.Hal",
-                                                  device_list[i],
-                                                  "org.freedesktop.Hal.Device.Synce");
-    if (proxy == NULL) {
-      g_critical("%s: Failed to get proxy for device '%s'", G_STRFUNC, device_list[i]);
-      goto error_exit;
-    }
-
-    if (!dbus_g_proxy_call(proxy, "RequestConnection", &error,
-                           G_TYPE_INVALID,
-                           G_TYPE_STRING, &unix_path,
-                           G_TYPE_INVALID))
-    {
-      g_critical("%s: Failed to get a connection for %s: %s: %s", G_STRFUNC, device_list[i], result->name, error->message);
-      g_object_unref(proxy);
-      goto error_exit;
-    }
-
-    g_object_unref(proxy);
-
-    result->fd = get_socket_from_dccm(unix_path);
-    g_free(unix_path);
-
-    if (result->fd < 0) {
-      g_critical("%s: Failed to get file-descriptor from dccm for %s", G_STRFUNC, device_list[i]);
-      goto error_exit;
-    }
-
     result->transport = g_strdup("hal");
 
     break;
@@ -610,13 +551,124 @@ void synce_info_destroy(SynceInfo* info)
 {
   if (info)
   {
-    FREE(info->ip);
+    FREE(info->device_ip);
+    FREE(info->local_iface_ip);
     FREE(info->password);
     FREE(info->name);
     FREE(info->os_name);
     FREE(info->model);
     FREE(info->transport);
+    FREE(info->object_path);
     free(info);
   }
 }
 
+const char *
+synce_info_get_name(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->name;
+}
+
+bool
+synce_info_get_os_version(SynceInfo *info, int *os_major, int *os_minor)
+{
+  if (!info) return NULL;
+  if ((!os_major) || (!os_minor))
+    return false;
+
+  *os_major = info->os_major;
+  *os_minor = info->os_minor;
+  return true;
+}
+
+int
+synce_info_get_build_number(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->build_number;
+}
+
+int
+synce_info_get_processor_type(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->processor_type;
+}
+
+const char *
+synce_info_get_os_name(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->os_name;
+}
+
+const char *
+synce_info_get_model(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->model;
+}
+
+const char *
+synce_info_get_device_ip(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->device_ip;
+}
+
+const char *
+synce_info_get_local_ip(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->local_iface_ip;
+}
+
+int
+synce_info_get_partner_id_1(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->partner_id_1;
+}
+
+int
+synce_info_get_partner_id_2(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->partner_id_2;
+}
+
+const char *
+synce_info_get_object_path(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->object_path;
+}
+
+pid_t
+synce_info_get_dccm_pid(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->dccm_pid;
+}
+
+const char *
+synce_info_get_transport(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->transport;
+}
+
+const char *
+synce_info_get_password(SynceInfo *info)
+{
+  if (!info) return NULL;
+  return info->password;
+}
+
+int
+synce_info_get_key(SynceInfo *info)
+{
+  if (!info) return 0;
+  return info->key;
+}
