@@ -1,4 +1,4 @@
-/* $Id: synce.h.in 3372 2008-04-11 09:47:16Z mark_ellis $ */
+/* $Id: synce.h.in 3780 2009-06-08 16:41:06Z mark_ellis $ */
 #ifndef __synce_h__
 #define __synce_h__
 
@@ -92,41 +92,57 @@ LPWSTR wstrdup(LPCWSTR string);
  * Endian conversions
  */
 
-#if !defined(htole32) || !defined(htole16) || !defined(letoh16) || !defined(letoh32)
+#if !defined(htole32) || !defined(htole16) || !defined(letoh16) || !defined(letoh32) || !defined(htobe32) || !defined(htobe16) || !defined(betoh16) || !defined(betoh32)
 #undef htole16
 #undef htole32
 #undef letoh16
 #undef letoh32
+#undef htobe16
+#undef htobe32
+#undef betoh16
+#undef betoh32
 
-/* define host-to-little-endian and little-endian-to-host macros */
-#if 0
+#define SYNCE_HOST_BIGENDIAN 0
+#define HAVE_BYTESWAP_HEADER 1
 
-/* byte swapping */
-#if 0
-#include <dummy>
+/* byte swapping functions */
+
+#if HAVE_BYTESWAP_HEADER
+#include <byteswap.h>
 #else
-#define IMPLEMENT_BSWAP_XX  1
 uint16_t bswap_16(uint16_t x);
 uint32_t bswap_32(uint32_t x);
-#endif
+#endif /* HAVE_BYTESWAP_HEADER */
 
-/* Use bswap_xx */
 
-#define htole16(x)		bswap_16(x)
-#define htole32(x)		bswap_32(x)
+/* define host-to-little-endian and little-endian-to-host macros */
+
+#if SYNCE_HOST_BIGENDIAN
+
+#define htole16(x)    bswap_16(x)
+#define htole32(x)    bswap_32(x)
 #define letoh16(x)    bswap_16(x)
 #define letoh32(x)    bswap_32(x)
 
-#else
+#define htobe16(x)    (x)
+#define htobe32(x)    (x)
+#define betoh16(x)    (x)
+#define betoh32(x)    (x)
 
-/* Empty macros */
+#else /* HOST_LITTLE_ENDIAN */
 
-#define htole16(x)		(x)
-#define htole32(x)		(x)
+#define htole16(x)    (x)
+#define htole32(x)    (x)
 #define letoh16(x)    (x)
 #define letoh32(x)    (x)
 
-#endif
+#define htobe16(x)    bswap_16(x)
+#define htobe32(x)    bswap_32(x)
+#define betoh16(x)    bswap_16(x)
+#define betoh32(x)    bswap_32(x)
+
+#endif /* SYNCE_HOST_BIGENDIAN */
+
 #endif
 
 
@@ -150,31 +166,32 @@ const char* synce_strerror(DWORD error);
 
 /*
  * Info from the ~/.synce/active_connection file
- * written by (v)dccm, or obtained from odccm
+ * written by (v)dccm, or obtained over dbus from odccm or hal-dccm
  */
 
-typedef struct _SynceInfo
-{
-  pid_t dccm_pid;
-  char* ip;
-  char* password;
-  int key;
-  int os_version;
-  int os_minor;
-  int build_number;
-  int processor_type;
-  int partner_id_1;
-  int partner_id_2;
-  char* name;
-  char* os_name;
-  char* model;
-  char* transport;
-  int fd;
-} SynceInfo;
+typedef struct _SynceInfo SynceInfo;
 
 SynceInfo* synce_info_new(const char* device_name);
 void synce_info_destroy(SynceInfo* info);
 
+const char *synce_info_get_name(SynceInfo *info);
+bool synce_info_get_os_version(SynceInfo *info, int *os_major, int *os_minor);
+int synce_info_get_build_number(SynceInfo *info);
+int synce_info_get_processor_type(SynceInfo *info);
+const char *synce_info_get_os_name(SynceInfo *info);
+const char *synce_info_get_model(SynceInfo *info);
+
+const char *synce_info_get_device_ip(SynceInfo *info);
+const char *synce_info_get_local_ip(SynceInfo *info);
+
+int synce_info_get_partner_id_1(SynceInfo *info);
+int synce_info_get_partner_id_2(SynceInfo *info);
+
+const char *synce_info_get_object_path(SynceInfo *info);
+pid_t synce_info_get_dccm_pid(SynceInfo *info);
+const char *synce_info_get_transport(SynceInfo *info);
+const char *synce_info_get_password(SynceInfo *info);
+int synce_info_get_key(SynceInfo *info);
 
 #ifdef __cplusplus
 }
